@@ -1,10 +1,10 @@
 package Data::Sah::Coerce;
 
 our $DATE = '2016-05-22'; # DATE
-our $VERSION = '0.009'; # VERSION
+our $VERSION = '0.010'; # VERSION
 
 use 5.010001;
-use strict 'subs', 'vars';
+use strict;
 use warnings;
 use Log::Any::IfLOG '$log';
 
@@ -17,91 +17,57 @@ our %SPEC;
 
 our $Log_Coercer_Code = $ENV{LOG_SAH_COERCER_CODE} // 0;
 
-my $rule_modules_cache;
-sub _list_rule_modules {
-    return $rule_modules_cache if $rule_modules_cache;
-    require PERLANCAR::Module::List;
-    my $prefix = "Data::Sah::Coerce::perl::";
-    my $mods = PERLANCAR::Module::List::list_modules(
-        $prefix, {list_modules=>1, recurse=>1},
-    );
-    $rule_modules_cache = $mods;
-    $mods;
-}
+$SPEC{gen_coercer} = {
+    v => 1.1,
+    summary => 'Generate coercer code',
+    description => <<'_',
 
-$SPEC{gen_coercer} = $Data::Sah::CoerceCommon::gen_coercer_meta;
+This is mostly for testing. Normally the coercion rules will be used from
+`Data::Sah`.
+
+_
+    args => {
+        %Data::Sah::CoerceCommon::gen_coercer_args,
+    },
+    result_naked => 1,
+};
 sub gen_coercer {
     my %args = @_;
 
-    my $type = $args{type};
     my $rt = $args{return_type} // 'val';
     my $rt_sv = $rt eq 'str+val';
 
-    my $all_mods = _list_rule_modules();
+    my $rules = Data::Sah::CoerceCommon::get_coerce_rules(
+        %args,
+        compiler=>'perl',
+        data_term=>'$data',
+    );
 
-    my $typen = $type; $typen =~ s/::/__/g;
-    my $prefix = "Data::Sah::Coerce::perl::$typen\::";
-    my @rules;
-    for my $mod (keys %$all_mods) {
-        next unless $mod =~ /\A\Q$prefix\E(.+)/;
-        push @rules, $1;
-    }
-    my %explicitly_included_rules;
-    for my $rule (@{ $args{coerce_from} // [] }) {
-        push @rules, $rule unless grep {$rule eq $_} @rules;
-        $explicitly_included_rules{$rule}++;
-    }
-    if ($args{dont_coerce_from} && @{ $args{dont_coerce_from} }) {
-        my @frules;
-        for my $rule (@rules) {
-            next if grep {$rule eq $_} @{ $args{dont_coerce_from} };
-            push @frules, $rule;
-        }
-        @rules = @frules;
-    }
-
-    my @res;
-    for my $rule (@rules) {
-        my $mod = "$prefix$rule";
-        my $mod_pm = $mod; $mod_pm =~ s!::!/!g; $mod_pm .= ".pm";
-        require $mod_pm;
-        next unless $explicitly_included_rules{$rule} ||
-            &{"$mod\::meta"}->{enable_by_default};
-        my $res = &{"$mod\::coerce"}(
-            data_term => '$data',
-            coerce_to => $args{coerce_to},
-        );
-        if ($res->{modules}) {
-            for my $mod (keys %{$res->{modules}}) {
+    my $code;
+    if (@$rules) {
+        # load all the required perl modules
+        for my $rule (@$rules) {
+            next unless $rule->{modules};
+            for my $mod (keys %{$rule->{modules}}) {
                 my $mod_pm = $mod; $mod_pm =~ s!::!/!g; $mod_pm .= ".pm";
                 require $mod_pm;
             }
         }
-        $res->{rule} = $rule;
-        push @res, $res;
-    }
-
-    my $code;
-    if (@res) {
-        @res = sort {
-            ($a->{meta}{prio}//50) <=> ($b->{meta}{prio}//50) ||
-                $a cmp $b
-            } @res;
 
         my $expr;
-        for my $i (reverse 0..$#res) {
-            my $res = $res[$i];
-            if ($i == $#res) {
+        for my $i (reverse 0..$#{$rules}) {
+            my $rule = $rules->[$i];
+            if ($i == $#{$rules}) {
                 if ($rt_sv) {
-                    $expr = "($res->{expr_match}) ? ['$res->{rule}', $res->{expr_coerce}] : [undef, \$data]";
+                    $expr = "($rule->{expr_match}) ? ['$rule->{name}', $rule->{expr_coerce}] : [undef, \$data]";
                 } else {
-                    $expr = "($res->{expr_match}) ? ($res->{expr_coerce}) : \$data";
+                    $expr = "($rule->{expr_match}) ? ($rule->{expr_coerce}) : \$data";
                 }
             } else {
                 if ($rt_sv) {
-                    $expr = "($res->{expr_match}) ? ['$res->{rule}', $res->{expr_coerce}] : ($expr)";
+                    $expr = "($rule->{expr_match}) ? ['$rule->{name}', $rule->{expr_coerce}] : ($expr)";
                 } else {
-                    $expr = "($res->{expr_match}) ? ($res->{expr_coerce}) : ($expr)";
+                    $expr = "($rule->{expr_match}) ? ($rule->{expr_coerce}) : ($expr)";
                 }
             }
         }
@@ -151,7 +117,7 @@ Data::Sah::Coerce - Coercion rules for Data::Sah
 
 =head1 VERSION
 
-This document describes version 0.009 of Data::Sah::Coerce (from Perl distribution Data-Sah-Coerce), released on 2016-05-22.
+This document describes version 0.010 of Data::Sah::Coerce (from Perl distribution Data-Sah-Coerce), released on 2016-05-22.
 
 =head1 SYNOPSIS
 
@@ -188,6 +154,11 @@ the following keys (C<*> marks that the key is required):
 
 =over
 
+=item * v* => int (default: 1)
+
+Metadata specification version. Currently at 2 (bumped from 1 to exclude old
+module names).
+
 =item * enable_by_default* => bool
 
 Whether the rule should be used by default. Some rules might be useful in
@@ -215,6 +186,36 @@ For a rule that might die, the program/library that uses the rule module might
 add an eval block around the expr_coerce code that is generated by the rule
 module.
 
+=item * prio => int (0-100, default: 50)
+
+This is to regulate the ordering of rules. The higher the number, the lower the
+priority (meaning the rule will be put further back). Rules that are
+computationally expensive and/or matches more broadly in general should be put
+further back (lower priority, higher number).
+
+=item * precludes => array of (str|re)
+
+List the other rules or rule patterns that are precluded by this rule. Rules
+that are pure alternatives to one another (e.g. date coercien rules
+L<str_alami|Data::Sah::Coerce::date::str_alami> vs
+L<str_alami|Data::Sah::Coerce::date::str_alami> both parse natural language date
+string; there is little to none usefulness in using both; besides, both rules
+match all string and dies when failing to parse the string. So in C<str_natural>
+rule, you might find this metadata:
+
+ precludes => [qr/\Astr_alami(_.+)?\z/]
+
+and in C<str_alami> rule you might find this metadata:
+
+ precludes => [qr/\Astr_alami(_.+)?\z/, 'str_natural']
+
+Note that the C<str_alami> rule also precludes other C<str_alami_*> rules (like
+C<str_alami_en> and C<str_alami_id>).
+
+Also note that rules which are specifically requested to be used (e.g. using
+C<x.perl.coerce_from> attribute in Sah schema) are not precluded by other rules'
+C<precludes> metadata.
+
 =back
 
 The module must also contain C<coerce> subroutine which must generate the code
@@ -292,7 +293,7 @@ something like:
 =head1 FUNCTIONS
 
 
-=head2 gen_coercer(%args) -> any
+=head2 gen_coercer() -> any
 
 Generate coercer code.
 
@@ -301,29 +302,7 @@ C<Data::Sah>.
 
 This function is not exported by default, but exportable.
 
-Arguments ('*' denotes required arguments):
-
-=over 4
-
-=item * B<coerce_from> => I<array[str]>
-
-=item * B<coerce_to> => I<str>
-
-=item * B<dont_coerce_from> => I<array[str]>
-
-=item * B<return_type> => I<str> (default: "val")
-
-C<val> returns the value (possibly) coerced. C<str+val> returns a 2-element array
-where the first element is a bool value of whether the value has been coerced,
-and the second element is the (possibly) coerced value.
-
-=item * B<source> => I<bool>
-
-If set to true, will return coercer source code string instead of compiled code.
-
-=item * B<type>* => I<str>
-
-=back
+No arguments.
 
 Return value:  (any)
 
